@@ -1,4 +1,4 @@
-# $Id: en.py 9120 2022-09-13 10:24:30Z milde $
+# $Id: en.py 9417 2023-06-27 20:04:54Z milde $
 # Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
@@ -26,7 +26,7 @@ directives = {
       'note': 'note',
       'tip': 'tip',
       'warning': 'warning',
-      'admonition': 'admonition',  # advice/caveat/remark, not reprimand
+      'admonition': 'admonition',  # advice/advisory/remark, not reprimand
       'sidebar': 'sidebar',
       'topic': 'topic',
       'line-block': 'line-block',
