@@ -1,36 +1,57 @@
-#! /usr/bin/env python
+#! /usr/bin/env python3
 
-# $Id: test_unicode.py 8481 2020-01-31 08:17:24Z milde $
+# $Id: test_unicode.py 9425 2023-06-30 14:56:47Z milde $
 # Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
 """
 Tests for misc.py "unicode" directive.
 """
-from __future__ import absolute_import
 
+
+from pathlib import Path
 import sys
+import unittest
 
 if __name__ == '__main__':
-    import __init__
-from test_parsers import DocutilsTestSupport
+    # prepend the "docutils root" to the Python library path
+    # so we import the local `docutils` package.
+    sys.path.insert(0, str(Path(__file__).resolve().parents[4]))
+
+from docutils.frontend import get_default_settings
+from docutils.parsers.rst import Parser
+from docutils.utils import new_document
 
 
-if sys.version_info >= (3, 0):
-    unichr = chr  # noqa
+class ParserTestCase(unittest.TestCase):
+    def test_parser(self):
+        parser = Parser()
+        settings = get_default_settings(Parser)
+        settings.warning_stream = ''
+        for name, cases in totest.items():
+            for casenum, (case_input, case_expected) in enumerate(cases):
+                with self.subTest(id=f'totest[{name!r}][{casenum}]'):
+                    document = new_document('test data', settings.copy())
+                    parser.parse(case_input, document)
+                    output = document.pformat()
+                    self.assertEqual(case_expected, output)
 
 
-def suite():
-    s = DocutilsTestSupport.ParserTestSuite()
-    s.generateTests(totest)
-    return s
-
-unichr_exception = DocutilsTestSupport.exception_data(
-    unichr, int("111111111111111111", 16))[0]
-if isinstance(unichr_exception, OverflowError):
-    unichr_exception_string = 'code too large (%s)' % unichr_exception
-else:
+try:
+    chr(0x111111111111111111)
+except OverflowError as unichr_exception:
+    unichr_exception_string = f'code too large ({unichr_exception})'
+except Exception as unichr_exception:
     unichr_exception_string = str(unichr_exception)
+else:
+    unichr_exception_string = ''
+
+try:
+    chr(0x11111111)
+except Exception as detail:
+    invalid_char_code = f'{detail.__class__.__name__}: {detail}'
+else:
+    invalid_char_code = ''
 
 totest = {}
 
@@ -45,7 +66,7 @@ space (|nbsp|), a backwards-not-equals (|bne|), and a captial omega (|Omega|).
 .. |bne| unicode:: U0003D U020E5
 .. |Omega| unicode:: U+003A9
 """,
-u"""\
+"""\
 <document source="test data">
     <paragraph>
         Insert an em-dash (
@@ -125,15 +146,15 @@ Copyright |copy| 2003, |BogusMegaCorp (TM)|.
 .. |BogusMegaCorp (TM)| unicode:: BogusMegaCorp U+2122
    .. with trademark sign
 """,
-u"""\
+"""\
 <document source="test data">
     <paragraph>
         Testing comments and extra text.
     <paragraph>
-        Copyright 
+        Copyright \n\
         <substitution_reference refname="copy">
             copy
-         2003, 
+         2003, \n\
         <substitution_reference refname="BogusMegaCorp (TM)">
             BogusMegaCorp (TM)
         .
@@ -171,11 +192,9 @@ u"""\
             Substitution definition "too big for unicode" empty or invalid.
         <literal_block xml:space="preserve">
             .. |too big for unicode| unicode:: 0x11111111
-""" % (unichr_exception_string,
-       DocutilsTestSupport.exception_data(unichr, int("11111111", 16))[2])]
+""" % (unichr_exception_string, invalid_char_code)]
 ]
 
 
 if __name__ == '__main__':
-    import unittest
-    unittest.main(defaultTest='suite')
+    unittest.main()
