@@ -1,7 +1,5 @@
-#! /usr/bin/env python
-# -*- coding: utf-8 -*-
-
-# $Id: test_inline_markup.py 8481 2020-01-31 08:17:24Z milde $
+#! /usr/bin/env python3
+# $Id: test_inline_markup.py 9425 2023-06-30 14:56:47Z milde $
 # Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
@@ -9,17 +7,34 @@
 Tests for inline markup in docutils/parsers/rst/states.py.
 Interpreted text tests are in a separate module, test_interpreted.py.
 """
-from __future__ import absolute_import
+
+from pathlib import Path
+import sys
+import unittest
 
 if __name__ == '__main__':
-    import __init__
-from test_parsers import DocutilsTestSupport
+    # prepend the "docutils root" to the Python library path
+    # so we import the local `docutils` package.
+    sys.path.insert(0, str(Path(__file__).resolve().parents[3]))
+
+from docutils.frontend import get_default_settings
+from docutils.parsers.rst import Parser
+from docutils.utils import new_document
 
 
-def suite():
-    s = DocutilsTestSupport.ParserTestSuite()
-    s.generateTests(totest)
-    return s
+class ParserTestCase(unittest.TestCase):
+    def test_parser(self):
+        parser = Parser()
+        settings = get_default_settings(Parser)
+        settings.warning_stream = ''
+        for name, cases in totest.items():
+            for casenum, (case_input, case_expected) in enumerate(cases):
+                with self.subTest(id=f'totest[{name!r}][{casenum}]'):
+                    document = new_document('test data', settings.copy())
+                    parser.parse(case_input, document)
+                    output = document.pformat()
+                    self.assertEqual(case_expected, output)
+
 
 totest = {}
 
@@ -33,11 +48,11 @@ totest['emphasis'] = [
         <emphasis>
             emphasis
 """],
-[u"""\
+["""\
 l'*emphasis* with the *emphasis*' apostrophe.
 l\u2019*emphasis* with the *emphasis*\u2019 apostrophe.
 """,
-u"""\
+"""\
 <document source="test data">
     <paragraph>
         l\'
@@ -72,10 +87,10 @@ across lines*
 """\
 <document source="test data">
     <paragraph>
-        <problematic ids="id2" refid="id1">
+        <problematic ids="problematic-1" refid="system-message-1">
             *
         emphasis without closing asterisk
-    <system_message backrefs="id2" ids="id1" level="2" line="1" source="test data" type="WARNING">
+    <system_message backrefs="problematic-1" ids="system-message-1" level="2" line="1" source="test data" type="WARNING">
         <paragraph>
             Inline emphasis start-string without end-string.
 """],
@@ -146,10 +161,10 @@ what about *this**?
          (escaped, whitespace before end-string).
     <paragraph>
         However, '
-        <problematic ids="id2" refid="id1">
+        <problematic ids="problematic-1" refid="system-message-1">
             *
         args' will trigger a warning and may be problematic.
-    <system_message backrefs="id2" ids="id1" level="2" line="13" source="test data" type="WARNING">
+    <system_message backrefs="problematic-1" ids="system-message-1" level="2" line="13" source="test data" type="WARNING">
         <paragraph>
             Inline emphasis start-string without end-string.
     <paragraph>
@@ -158,7 +173,7 @@ what about *this**?
             this*
         ?
 """],
-[u"""\
+["""\
 Quotes around inline markup:
 
 '*emphasis*' "*emphasis*" Straight,
@@ -173,7 +188,7 @@ Quotes around inline markup:
 „*emphasis*” ‚*emphasis*’ Polish,
 „*emphasis*” »*emphasis*« ’*emphasis*’ Hungarian,
 """,
-u"""\
+"""\
 <document source="test data">
     <paragraph>
         Quotes around inline markup:
@@ -305,10 +320,10 @@ totest['strong'] = [
         <strong>
             strong
 """],
-[u"""\
+["""\
 l'**strong** and l\u2019**strong** with apostrophe
 """,
-u"""\
+"""\
 <document source="test data">
     <paragraph>
         l'
@@ -319,12 +334,12 @@ u"""\
             strong
          with apostrophe
 """],
-[u"""\
+["""\
 quoted '**strong**', quoted "**strong**",
 quoted \u2018**strong**\u2019, quoted \u201c**strong**\u201d,
 quoted \xab**strong**\xbb
 """,
-u"""\
+"""\
 <document source="test data">
     <paragraph>
         quoted '
@@ -360,10 +375,10 @@ u"""\
         ) but not (**) or '(** ' or x**2 or **kwargs or **
     <paragraph>
         (however, '
-        <problematic ids="id2" refid="id1">
+        <problematic ids="problematic-1" refid="system-message-1">
             **
         kwargs' will trigger a warning and may be problematic)
-    <system_message backrefs="id2" ids="id1" level="2" line="4" source="test data" type="WARNING">
+    <system_message backrefs="problematic-1" ids="system-message-1" level="2" line="4" source="test data" type="WARNING">
         <paragraph>
             Inline strong start-string without end-string.
 """],
@@ -389,10 +404,10 @@ Strong double asterisk: ******
 """\
 <document source="test data">
     <paragraph>
-        <problematic ids="id2" refid="id1">
+        <problematic ids="problematic-1" refid="system-message-1">
             **
         strong without closing asterisks
-    <system_message backrefs="id2" ids="id1" level="2" line="1" source="test data" type="WARNING">
+    <system_message backrefs="problematic-1" ids="system-message-1" level="2" line="1" source="test data" type="WARNING">
         <paragraph>
             Inline strong start-string without end-string.
 """],
@@ -435,10 +450,10 @@ totest['literal'] = [
         <literal>
             literal\\
 """],
-[u"""\
+["""\
 l'``literal`` and l\u2019``literal`` with apostrophe
 """,
-u"""\
+"""\
 <document source="test data">
     <paragraph>
         l'
@@ -449,12 +464,12 @@ u"""\
             literal
          with apostrophe
 """],
-[u"""\
+["""\
 quoted '``literal``', quoted "``literal``",
 quoted \u2018``literal``\u2019, quoted \u201c``literal``\u201d,
 quoted \xab``literal``\xbb
 """,
-u"""\
+"""\
 <document source="test data">
     <paragraph>
         quoted '
@@ -476,12 +491,12 @@ u"""\
             literal
         \xbb
 """],
-[u"""\
+["""\
 ``'literal'`` with quotes, ``"literal"`` with quotes,
 ``\u2018literal\u2019`` with quotes, ``\u201cliteral\u201d`` with quotes,
 ``\xabliteral\xbb`` with quotes
 """,
-u"""\
+"""\
 <document source="test data">
     <paragraph>
         <literal>
@@ -514,11 +529,11 @@ and may be problematic)
          but not "``" or ``
     <paragraph>
         (however, \n\
-        <problematic ids="id2" refid="id1">
+        <problematic ids="problematic-1" refid="system-message-1">
             ``
         standalone TeX quotes'' will trigger a warning
         and may be problematic)
-    <system_message backrefs="id2" ids="id1" level="2" line="4" source="test data" type="WARNING">
+    <system_message backrefs="problematic-1" ids="system-message-1" level="2" line="4" source="test data" type="WARNING">
         <paragraph>
             Inline literal start-string without end-string.
 """],
@@ -539,10 +554,10 @@ Find the ```interpreted text``` in this paragraph!
 """\
 <document source="test data">
     <paragraph>
-        <problematic ids="id2" refid="id1">
+        <problematic ids="problematic-1" refid="system-message-1">
             ``
         literal without closing backquotes
-    <system_message backrefs="id2" ids="id1" level="2" line="1" source="test data" type="WARNING">
+    <system_message backrefs="problematic-1" ids="system-message-1" level="2" line="1" source="test data" type="WARNING">
         <paragraph>
             Inline literal start-string without end-string.
 """],
@@ -599,10 +614,10 @@ ref_
         <reference name="ref" refname="ref">
             ref
 """],
-[u"""\
+["""\
 l'ref_ and l\u2019ref_ with apostrophe
 """,
-u"""\
+"""\
 <document source="test data">
     <paragraph>
         l'
@@ -613,14 +628,14 @@ u"""\
             ref
          with apostrophe
 """],
-[u"""\
+["""\
 quoted 'ref_', quoted "ref_",
 quoted \u2018ref_\u2019, quoted \u201cref_\u201d,
 quoted \xabref_\xbb,
 but not 'ref ref'_, "ref ref"_, \u2018ref ref\u2019_,
 \u201cref ref\u201d_, or \xabref ref\xbb_
 """,
-u"""\
+"""\
 <document source="test data">
     <paragraph>
         quoted '
@@ -653,10 +668,10 @@ ref__
         <reference anonymous="1" name="ref">
             ref
 """],
-[u"""\
+["""\
 l'ref__ and l\u2019ref__ with apostrophe
 """,
-u"""\
+"""\
 <document source="test data">
     <paragraph>
         l'
@@ -667,14 +682,14 @@ u"""\
             ref
          with apostrophe
 """],
-[u"""\
+["""\
 quoted 'ref__', quoted "ref__",
 quoted \u2018ref__\u2019, quoted \u201cref__\u201d,
 quoted \xabref__\xbb,
 but not 'ref ref'__, "ref ref"__, \u2018ref ref\u2019__,
 \u201cref ref\u201d__, or \xabref ref\xbb__
 """,
-u"""\
+"""\
 <document source="test data">
     <paragraph>
         quoted '
@@ -734,10 +749,10 @@ totest['phrase_references'] = [
         <reference name="phrase reference" refname="phrase reference">
             phrase reference
 """],
-[u"""\
+["""\
 l'`phrase reference`_ and l\u2019`phrase reference`_ with apostrophe
 """,
-u"""\
+"""\
 <document source="test data">
     <paragraph>
         l'
@@ -748,13 +763,13 @@ u"""\
             phrase reference
          with apostrophe
 """],
-[u"""\
+["""\
 quoted '`phrase reference`_', quoted "`phrase reference`_",
 quoted \u2018`phrase reference`_\u2019,
 quoted \u201c`phrase reference`_\u201d,
 quoted \xab`phrase reference`_\xbb
 """,
-u"""\
+"""\
 <document source="test data">
     <paragraph>
         quoted '
@@ -777,13 +792,13 @@ u"""\
             phrase reference
         \xbb
 """],
-[u"""\
+["""\
 `'phrase reference'`_ with quotes, `"phrase reference"`_ with quotes,
 `\u2018phrase reference\u2019`_ with quotes,
 `\u201cphrase reference\u201d`_ with quotes,
 `\xabphrase reference\xbb`_ with quotes
 """,
-u"""\
+"""\
 <document source="test data">
     <paragraph>
         <reference name="'phrase reference'" refname="'phrase reference'">
@@ -811,10 +826,10 @@ u"""\
         <reference anonymous="1" name="anonymous reference">
             anonymous reference
 """],
-[u"""\
+["""\
 l'`anonymous reference`__ and l\u2019`anonymous reference`__ with apostrophe
 """,
-u"""\
+"""\
 <document source="test data">
     <paragraph>
         l'
@@ -825,13 +840,13 @@ u"""\
             anonymous reference
          with apostrophe
 """],
-[u"""\
+["""\
 quoted '`anonymous reference`__', quoted "`anonymous reference`__",
 quoted \u2018`anonymous reference`__\u2019,
 quoted \u201c`anonymous reference`__\u201d,
 quoted \xab`anonymous reference`__\xbb
 """,
-u"""\
+"""\
 <document source="test data">
     <paragraph>
         quoted '
@@ -854,13 +869,13 @@ u"""\
             anonymous reference
         \xbb
 """],
-[u"""\
+["""\
 `'anonymous reference'`__ with quotes, `"anonymous reference"`__ with quotes,
 `\u2018anonymous reference\u2019`__ with quotes,
 `\u201canonymous reference\u201d`__ with quotes,
 `\xabanonymous reference\xbb`__ with quotes
 """,
-u"""\
+"""\
 <document source="test data">
     <paragraph>
         <reference anonymous="1" name="'anonymous reference'">
@@ -909,9 +924,9 @@ Invalid phrase reference:
     <paragraph>
         Invalid phrase reference:
     <paragraph>
-        <problematic ids="id2" refid="id1">
+        <problematic ids="problematic-1" refid="system-message-1">
             :role:`phrase reference`_
-    <system_message backrefs="id2" ids="id1" level="2" line="3" source="test data" type="WARNING">
+    <system_message backrefs="problematic-1" ids="system-message-1" level="2" line="3" source="test data" type="WARNING">
         <paragraph>
             Mismatch: both interpreted text role prefix and reference suffix.
 """],
@@ -925,9 +940,9 @@ Invalid phrase reference:
     <paragraph>
         Invalid phrase reference:
     <paragraph>
-        <problematic ids="id2" refid="id1">
+        <problematic ids="problematic-1" refid="system-message-1">
             `phrase reference`:role:_
-    <system_message backrefs="id2" ids="id1" level="2" line="3" source="test data" type="WARNING">
+    <system_message backrefs="problematic-1" ids="system-message-1" level="2" line="3" source="test data" type="WARNING">
         <paragraph>
             Mismatch: both interpreted text role suffix and reference suffix.
 """],
@@ -937,13 +952,13 @@ Invalid phrase reference:
 """\
 <document source="test data">
     <paragraph>
-        <problematic ids="id2" refid="id1">
+        <problematic ids="problematic-1" refid="system-message-1">
             `
         phrase \n\
         <reference name="reference" refname="reference">
             reference
          without closing backquote
-    <system_message backrefs="id2" ids="id1" level="2" line="1" source="test data" type="WARNING">
+    <system_message backrefs="problematic-1" ids="system-message-1" level="2" line="1" source="test data" type="WARNING">
         <paragraph>
             Inline interpreted text or phrase reference start-string without end-string.
 """],
@@ -953,19 +968,19 @@ Invalid phrase reference:
 """\
 <document source="test data">
     <paragraph>
-        <problematic ids="id2" refid="id1">
+        <problematic ids="problematic-1" refid="system-message-1">
             `
         anonymous phrase \n\
         <reference anonymous="1" name="reference">
             reference
          without closing backquote
-    <system_message backrefs="id2" ids="id1" level="2" line="1" source="test data" type="WARNING">
+    <system_message backrefs="problematic-1" ids="system-message-1" level="2" line="1" source="test data" type="WARNING">
         <paragraph>
             Inline interpreted text or phrase reference start-string without end-string.
 """],
 ]
 
-totest['embedded_URIs'] = [
+totest['embedded_uris'] = [
 ["""\
 `phrase reference <http://example.com>`_
 """,
@@ -1180,7 +1195,7 @@ phrase_>`__
             embedded alias across lines
 """],
 ["""\
-`embedded alias with whitespace <alias 
+`embedded alias with whitespace <alias
 long  phrase_>`__
 """,
 """\
@@ -1260,10 +1275,10 @@ _`Here is  a    TaRgeT` with case and spacial difficulties.
             Here is  a    TaRgeT
          with case and spacial difficulties.
 """],
-[u"""\
+["""\
 l'_`target1` and l\u2019_`target2` with apostrophe
 """,
-u"""\
+"""\
 <document source="test data">
     <paragraph>
         l'
@@ -1274,12 +1289,12 @@ u"""\
             target2
          with apostrophe
 """],
-[u"""\
+["""\
 quoted '_`target1`', quoted "_`target2`",
 quoted \u2018_`target3`\u2019, quoted \u201c_`target4`\u201d,
 quoted \xab_`target5`\xbb
 """,
-u"""\
+"""\
 <document source="test data">
     <paragraph>
         quoted '
@@ -1301,12 +1316,12 @@ u"""\
             target5
         \xbb
 """],
-[u"""\
+["""\
 _`'target1'` with quotes, _`"target2"` with quotes,
 _`\u2018target3\u2019` with quotes, _`\u201ctarget4\u201d` with quotes,
 _`\xabtarget5\xbb` with quotes
 """,
-u"""\
+"""\
 <document source="test data">
     <paragraph>
         <target ids="target1" names="'target1'">
@@ -1336,10 +1351,10 @@ And _`this`_ is just plain confusing.
         But this isn't a _target; targets require backquotes.
     <paragraph>
         And \n\
-        <problematic ids="id2" refid="id1">
+        <problematic ids="problematic-1" refid="system-message-1">
             _`
         this`_ is just plain confusing.
-    <system_message backrefs="id2" ids="id1" level="2" line="3" source="test data" type="WARNING">
+    <system_message backrefs="problematic-1" ids="system-message-1" level="2" line="3" source="test data" type="WARNING">
         <paragraph>
             Inline target start-string without end-string.
 """],
@@ -1349,10 +1364,10 @@ _`inline target without closing backquote
 """\
 <document source="test data">
     <paragraph>
-        <problematic ids="id2" refid="id1">
+        <problematic ids="problematic-1" refid="system-message-1">
             _`
         inline target without closing backquote
-    <system_message backrefs="id2" ids="id1" level="2" line="1" source="test data" type="WARNING">
+    <system_message backrefs="problematic-1" ids="system-message-1" level="2" line="1" source="test data" type="WARNING">
         <paragraph>
             Inline target start-string without end-string.
 """],
@@ -1365,7 +1380,7 @@ totest['footnote_reference'] = [
 """\
 <document source="test data">
     <paragraph>
-        <footnote_reference ids="id1" refname="1">
+        <footnote_reference ids="footnote-reference-1" refname="1">
             1
 """],
 ["""\
@@ -1374,7 +1389,7 @@ totest['footnote_reference'] = [
 """\
 <document source="test data">
     <paragraph>
-        <footnote_reference auto="1" ids="id1">
+        <footnote_reference auto="1" ids="footnote-reference-1">
 """],
 ["""\
 [#label]_
@@ -1382,7 +1397,7 @@ totest['footnote_reference'] = [
 """\
 <document source="test data">
     <paragraph>
-        <footnote_reference auto="1" ids="id1" refname="label">
+        <footnote_reference auto="1" ids="footnote-reference-1" refname="label">
 """],
 ["""\
 [*]_
@@ -1390,7 +1405,7 @@ totest['footnote_reference'] = [
 """\
 <document source="test data">
     <paragraph>
-        <footnote_reference auto="*" ids="id1">
+        <footnote_reference auto="*" ids="footnote-reference-1">
 """],
 ["""\
 Adjacent footnote refs are not possible: [*]_[#label]_ [#]_[2]_ [1]_[*]_
@@ -1409,7 +1424,7 @@ totest['citation_reference'] = [
 """\
 <document source="test data">
     <paragraph>
-        <citation_reference ids="id1" refname="citation">
+        <citation_reference ids="citation-reference-1" refname="citation">
             citation
 """],
 ["""\
@@ -1418,16 +1433,16 @@ totest['citation_reference'] = [
 """\
 <document source="test data">
     <paragraph>
-        <citation_reference ids="id1" refname="citation">
+        <citation_reference ids="citation-reference-1" refname="citation">
             citation
          and \n\
-        <citation_reference ids="id2" refname="cit-ation">
+        <citation_reference ids="citation-reference-2" refname="cit-ation">
             cit-ation
          and \n\
-        <citation_reference ids="id3" refname="cit.ation">
+        <citation_reference ids="citation-reference-3" refname="cit.ation">
             cit.ation
          and \n\
-        <citation_reference ids="id4" refname="cit1">
+        <citation_reference ids="citation-reference-4" refname="cit1">
             CIT1
          but not [CIT 1]_
 """],
@@ -1491,10 +1506,10 @@ reference|
 """\
 <document source="test data">
     <paragraph>
-        <problematic ids="id2" refid="id1">
+        <problematic ids="problematic-1" refid="system-message-1">
             |
         substitution reference without closing verbar
-    <system_message backrefs="id2" ids="id1" level="2" line="1" source="test data" type="WARNING">
+    <system_message backrefs="problematic-1" ids="system-message-1" level="2" line="1" source="test data" type="WARNING">
         <paragraph>
             Inline substitution_reference start-string without end-string.
 """],
@@ -1506,6 +1521,23 @@ first | then || and finally |||
     <paragraph>
         first | then || and finally |||
 """],
+["""\
+Substitution reference with |mix|.
+
+.. |mix| replace:: text and hyperlink-reference_
+""",
+"""\
+<document source="test data">
+    <paragraph>
+        Substitution reference with \n\
+        <substitution_reference refname="mix">
+            mix
+        .
+    <substitution_definition names="mix">
+        text and \n\
+        <reference name="hyperlink-reference" refname="hyperlink-reference">
+            hyperlink-reference
+"""],
 ]
 
 totest['standalone_hyperlink'] = [
@@ -1656,7 +1688,7 @@ Escaped email addresses are not recognized: test\\@example.org
 """],
 ]
 
-totest['markup recognition rules'] = [
+totest['markup_recognition_rules'] = [
 ["""\
 __This__ should be left alone.
 """,
@@ -1685,11 +1717,11 @@ lines.
         p
         with backslash-escaped whitespace, including newlines.
 """],
-[u"""\
+["""\
 text-*separated*\u2010*by*\u2011*various*\u2012*dashes*\u2013*and*\u2014*hyphens*.
 \u00bf*punctuation*? \u00a1*examples*!\u00a0*no-break-space*\u00a0.
 """,
-u"""\
+"""\
 <document source="test data">
     <paragraph>
         text-
@@ -1724,7 +1756,7 @@ u"""\
 """],
 # Whitespace characters:
 #  \u180e*MONGOLIAN VOWEL SEPARATOR*\u180e,   fails in Python 2.6
-[u"""\
+["""\
 text separated by
 *newline*
 or *space* or one of
@@ -1746,7 +1778,7 @@ or *space* or one of
 \u3000*IDEOGRAPHIC SPACE*\u3000,
 \u2028*LINE SEPARATOR*\u2028
 """,
-u"""\
+"""\
 <document source="test data">
     <paragraph>
         text separated by
@@ -1825,13 +1857,13 @@ u"""\
         <emphasis>
             LINE SEPARATOR
 """],
-[u"""\
+["""\
 inline markup separated by non-ASCII whitespace
 \xa0**NO-BREAK SPACE**\xa0, \xa0``NO-BREAK SPACE``\xa0, \xa0`NO-BREAK SPACE`\xa0,
 \u2000**EN QUAD**\u2000, \u2000``EN QUAD``\u2000, \u2000`EN QUAD`\u2000,
 \u202f**NARROW NBSP**\u202f, \u202f``NARROW NBSP``\u202f, \u202f`NARROW NBSP`\u202f,
-""",                                      
-u"""\
+""",
+"""\
 <document source="test data">
     <paragraph>
         inline markup separated by non-ASCII whitespace
@@ -1866,7 +1898,7 @@ u"""\
             NARROW NBSP
         \u202f,
 """],
-[u"""\
+["""\
 no inline markup due to whitespace inside and behind: *
 newline
 *
@@ -1889,7 +1921,7 @@ newline
 *\u3000IDEOGRAPHIC SPACE\u3000*
 *\u2028LINE SEPARATOR\u2028*
 """,
-u"""\
+"""\
 <document source="test data">
     <paragraph>
         no inline markup due to whitespace inside and behind: *
@@ -1914,23 +1946,24 @@ u"""\
         *\u3000IDEOGRAPHIC SPACE\u3000*
         *
         LINE SEPARATOR
-        *"""],
-[u"""\
+        *
+"""],
+["""\
 no inline markup because of non-ASCII whitespace following /preceding the markup
 **\xa0NO-BREAK SPACE\xa0** ``\xa0NO-BREAK SPACE\xa0`` `\xa0NO-BREAK SPACE\xa0`
 **\u2000EN QUAD\u2000** ``\u2000EN QUAD\u2000`` `\u2000EN QUAD\u2000`
 **\u202fNARROW NBSP\u202f** ``\u202fNARROW NBSP\u202f`` `\u202fNARROW NBSP\u202f`
 """,
-u"""\
+"""\
 <document source="test data">
     <paragraph>
         no inline markup because of non-ASCII whitespace following /preceding the markup
         **\xa0NO-BREAK SPACE\xa0** ``\xa0NO-BREAK SPACE\xa0`` `\xa0NO-BREAK SPACE\xa0`
         **\u2000EN QUAD\u2000** ``\u2000EN QUAD\u2000`` `\u2000EN QUAD\u2000`
-        **\u202fNARROW NBSP\u202f** ``\u202fNARROW NBSP\u202f`` `\u202fNARROW NBSP\u202f`\
+        **\u202fNARROW NBSP\u202f** ``\u202fNARROW NBSP\u202f`` `\u202fNARROW NBSP\u202f`
 """],
 # « * » ‹ * › « * » ‹ * › « * » ‹ * › French,
-[u"""\
+["""\
 "Quoted" markup start-string (matched openers & closers) -> no markup:
 
 '*' "*" (*) <*> [*] {*}
@@ -1948,7 +1981,7 @@ Some international quoting styles:
 
 But this is „*’ emphasized »*‹.
 """,
-u"""\
+"""\
 <document source="test data">
     <paragraph>
         "Quoted" markup start-string (matched openers & closers) -> no markup:
@@ -1975,5 +2008,4 @@ u"""\
 
 
 if __name__ == '__main__':
-    import unittest
-    unittest.main(defaultTest='suite')
+    unittest.main()
