@@ -1,6 +1,6 @@
 #! /usr/bin/env python3
 
-# $Id: test_transitions.py 9277 2022-11-26 23:15:13Z milde $
+# $Id: test_transitions.py 9425 2023-06-30 14:56:47Z milde $
 # Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
@@ -34,7 +34,7 @@ class ParserTestCase(unittest.TestCase):
                     document = new_document('test data', settings.copy())
                     parser.parse(case_input, document)
                     output = document.pformat()
-                    self.assertEqual(output, case_expected)
+                    self.assertEqual(case_expected, output)
 
 
 totest = {}
