@@ -1,6 +1,6 @@
 #! /usr/bin/env python3
 
-# $Id: test_hyperlinks.py 9037 2022-03-05 23:31:10Z milde $
+# $Id: test_hyperlinks.py 9425 2023-06-30 14:56:47Z milde $
 # Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
@@ -8,20 +8,41 @@
 Tests for docutils.transforms.references.Hyperlinks.
 """
 
+from pathlib import Path
+import sys
+import unittest
+
 if __name__ == '__main__':
-    import __init__  # noqa: F401
-from test_transforms import DocutilsTestSupport
+    # prepend the "docutils root" to the Python library path
+    # so we import the local `docutils` package.
+    sys.path.insert(0, str(Path(__file__).resolve().parents[2]))
+
+from docutils.frontend import get_default_settings
+from docutils.parsers.rst import Parser
 from docutils.transforms.references import PropagateTargets, \
      AnonymousHyperlinks, IndirectHyperlinks, ExternalTargets, \
      InternalTargets, DanglingReferences
-from docutils.parsers.rst import Parser
+from docutils.transforms.universal import TestMessages
+from docutils.utils import new_document
 
 
-def suite():
-    parser = Parser()
-    s = DocutilsTestSupport.TransformTestSuite(parser)
-    s.generateTests(totest)
-    return s
+class TransformTestCase(unittest.TestCase):
+    def test_transforms(self):
+        parser = Parser()
+        settings = get_default_settings(Parser)
+        settings.warning_stream = ''
+        for name, (transforms, cases) in totest.items():
+            for casenum, (case_input, case_expected) in enumerate(cases):
+                with self.subTest(id=f'totest[{name!r}][{casenum}]'):
+                    document = new_document('test data', settings.copy())
+                    parser.parse(case_input, document)
+                    # Don't do a ``populate_from_components()`` because that
+                    # would enable the Transformer's default transforms.
+                    document.transformer.add_transforms(transforms)
+                    document.transformer.add_transform(TestMessages)
+                    document.transformer.apply_transforms()
+                    output = document.pformat()
+                    self.assertEqual(case_expected, output)
 
 
 totest = {}
@@ -394,7 +415,7 @@ An `embedded alias <alias_>`_ with unknown reference.
             Unknown target name: "alias".
     <system_message level="1" line="1" source="test data" type="INFO">
         <paragraph>
-            Hyperlink target "embedded alias" is not referenced.\
+            Hyperlink target "embedded alias" is not referenced.
 """],
 ["""\
 An embedded URI with trailing underline:
@@ -937,5 +958,4 @@ Duplicate manual footnote labels, with reference ([1]_):
 
 
 if __name__ == '__main__':
-    import unittest
-    unittest.main(defaultTest='suite')
+    unittest.main()
